@@ -1,5 +1,4 @@
 """Persistent Data Types
-
 """
 
 import operator as op
@@ -18,10 +17,10 @@ from .core import ENOVAL, Cache
 
 
 def _make_compare(seq_op, doc):
-    "Make compare method with Sequence semantics."
+    """Make compare method with Sequence semantics."""
 
     def compare(self, that):
-        "Compare method for deque and sequence."
+        """Compare method for deque and sequence."""
         if not isinstance(that, Sequence):
             return NotImplemented
 
@@ -117,12 +116,12 @@ class Deque(Sequence):
 
     @property
     def cache(self):
-        "Cache used by deque."
+        """Cache used by deque."""
         return self._cache
 
     @property
     def directory(self):
-        "Directory path where deque is stored."
+        """Directory path where deque is stored."""
         return self._cache.directory
 
     def _index(self, index, func):
@@ -699,12 +698,12 @@ class Index(MutableMapping):
 
     @property
     def cache(self):
-        "Cache used by index."
+        """Cache used by index."""
         return self._cache
 
     @property
     def directory(self):
-        "Directory path where items are stored."
+        """Directory path where items are stored."""
         return self._cache.directory
 
     def __getitem__(self, key):
@@ -1097,7 +1096,7 @@ class Index(MutableMapping):
         """
         return not self == other
 
-    def memoize(self, name=None, typed=False):
+    def memoize(self, name=None, typed=False, ignore=()):
         """Memoizing cache decorator.
 
         Decorator to wrap callable with memoizing function using cache.
@@ -1148,10 +1147,11 @@ class Index(MutableMapping):
 
         :param str name: name given for callable (default None, automatic)
         :param bool typed: cache different types separately (default False)
+        :param set ignore: positional or keyword args to ignore (default ())
         :return: callable decorator
 
         """
-        return self._cache.memoize(name, typed)
+        return self._cache.memoize(name, typed, ignore=ignore)
 
     @contextmanager
     def transact(self):
