@@ -1,4 +1,4 @@
-"Test diskcache.core.Cache."
+"""Test diskcache.core.Cache."""
 
 import errno
 import hashlib
@@ -89,6 +89,11 @@ def test_custom_disk():
         for value in values:
             assert cache[value] == value
 
+        for key, value in zip(cache, values):
+            assert key == value
+
+        test_memoize_iter(cache)
+
     shutil.rmtree(cache.directory, ignore_errors=True)
 
 
@@ -333,26 +338,6 @@ def test_get_expired_slow_path(cache):
     assert cache.get(0) is None
 
 
-def test_get_ioerror_slow_path(cache):
-    cache.reset('eviction_policy', 'least-recently-used')
-    cache.set(0, 0)
-
-    disk = mock.Mock()
-    put = mock.Mock()
-    fetch = mock.Mock()
-
-    disk.put = put
-    put.side_effect = [(0, True)]
-    disk.fetch = fetch
-    io_error = IOError()
-    io_error.errno = errno.EACCES
-    fetch.side_effect = io_error
-
-    with mock.patch.object(cache, '_disk', disk):
-        with pytest.raises(IOError):
-            cache.get(0)
-
-
 def test_pop(cache):
     assert cache.incr('alpha') == 1
     assert cache.pop('alpha') == 1
@@ -396,25 +381,6 @@ def test_pop_ioerror(cache):
         assert cache.pop(0) is None
 
 
-def test_pop_ioerror_eacces(cache):
-    assert cache.set(0, 0)
-
-    disk = mock.Mock()
-    put = mock.Mock()
-    fetch = mock.Mock()
-
-    disk.put = put
-    put.side_effect = [(0, True)]
-    disk.fetch = fetch
-    io_error = IOError()
-    io_error.errno = errno.EACCES
-    fetch.side_effect = io_error
-
-    with mock.patch.object(cache, '_disk', disk):
-        with pytest.raises(IOError):
-            cache.pop(0)
-
-
 def test_delete(cache):
     cache[0] = 0
     assert cache.delete(0)
@@ -591,29 +557,6 @@ def test_least_frequently_used(cache):
     assert len(cache.check()) == 0
 
 
-def test_filename_error(cache):
-    func = mock.Mock(side_effect=OSError(errno.EACCES))
-
-    with mock.patch('os.makedirs', func):
-        with pytest.raises(OSError):
-            cache._disk.filename()
-
-
-def test_remove_error(cache):
-    func = mock.Mock(side_effect=OSError(errno.EACCES))
-
-    try:
-        with mock.patch('os.remove', func):
-            cache._disk.remove('ab/cd/efg.val')
-    except OSError:
-        pass
-    else:
-        if os.name == 'nt':
-            pass  # File delete errors ignored on Windows.
-        else:
-            raise Exception('test_remove_error failed')
-
-
 def test_check(cache):
     blob = b'a' * 2 ** 20
     keys = (0, 1, 1234, 56.78, u'hello', b'world', None)
@@ -1028,44 +971,6 @@ def test_peek_ioerror(cache):
         assert value == 0
 
 
-def test_pull_ioerror_eacces(cache):
-    assert cache.push(0) == 500000000000000
-
-    disk = mock.Mock()
-    put = mock.Mock()
-    fetch = mock.Mock()
-
-    disk.put = put
-    put.side_effect = [(0, True)]
-    disk.fetch = fetch
-    io_error = IOError()
-    io_error.errno = errno.EACCES
-    fetch.side_effect = io_error
-
-    with mock.patch.object(cache, '_disk', disk):
-        with pytest.raises(IOError):
-            cache.pull()
-
-
-def test_peek_ioerror_eacces(cache):
-    assert cache.push(0) == 500000000000000
-
-    disk = mock.Mock()
-    put = mock.Mock()
-    fetch = mock.Mock()
-
-    disk.put = put
-    put.side_effect = [(0, True)]
-    disk.fetch = fetch
-    io_error = IOError()
-    io_error.errno = errno.EACCES
-    fetch.side_effect = io_error
-
-    with mock.patch.object(cache, '_disk', disk):
-        with pytest.raises(IOError):
-            cache.peek()
-
-
 def test_peekitem_extras(cache):
     with pytest.raises(KeyError):
         cache.peekitem()
@@ -1117,27 +1022,6 @@ def test_peekitem_ioerror(cache):
         assert value == 2
 
 
-def test_peekitem_ioerror_eacces(cache):
-    assert cache.set('a', 0)
-    assert cache.set('b', 1)
-    assert cache.set('c', 2)
-
-    disk = mock.Mock()
-    put = mock.Mock()
-    fetch = mock.Mock()
-
-    disk.put = put
-    put.side_effect = [(0, True)]
-    disk.fetch = fetch
-    io_error = IOError()
-    io_error.errno = errno.EACCES
-    fetch.side_effect = io_error
-
-    with mock.patch.object(cache, '_disk', disk):
-        with pytest.raises(IOError):
-            cache.peekitem()
-
-
 def test_iterkeys(cache):
     assert list(cache.iterkeys()) == []
 
@@ -1459,3 +1343,57 @@ def test_memoize(cache):
 
     assert hits2 == (hits1 + count)
     assert misses2 == misses1
+
+
+def test_memoize_kwargs(cache):
+    @cache.memoize(typed=True)
+    def foo(*args, **kwargs):
+        return args, kwargs
+
+    assert foo(1, 2, 3, a=4, b=5) == ((1, 2, 3), {'a': 4, 'b': 5})
+
+
+def test_cleanup_dirs(cache):
+    value = b'\0' * 2 ** 20
+    start_count = len(os.listdir(cache.directory))
+    for i in range(10):
+        cache[i] = value
+    set_count = len(os.listdir(cache.directory))
+    assert set_count > start_count
+    for i in range(10):
+        del cache[i]
+    del_count = len(os.listdir(cache.directory))
+    assert start_count == del_count
+
+
+def test_disk_write_os_error(cache):
+    func = mock.Mock(side_effect=[OSError] * 10)
+    with mock.patch('diskcache.core.open', func):
+        with pytest.raises(OSError):
+            cache[0] = '\0' * 2 ** 20
+
+
+def test_memoize_ignore(cache):
+    @cache.memoize(ignore={1, 'arg1'})
+    def test(*args, **kwargs):
+        return args, kwargs
+
+    cache.stats(enable=True)
+    assert test('a', 'b', 'c', arg0='d', arg1='e', arg2='f')
+    assert test('a', 'w', 'c', arg0='d', arg1='x', arg2='f')
+    assert test('a', 'y', 'c', arg0='d', arg1='z', arg2='f')
+    assert cache.stats() == (2, 1)
+
+
+def test_memoize_iter(cache):
+    @cache.memoize()
+    def test(*args, **kwargs):
+        return sum(args) + sum(kwargs.values())
+
+    cache.clear()
+    assert test(1, 2, 3)
+    assert test(a=1, b=2, c=3)
+    assert test(-1, 0, 1, a=1, b=2, c=3)
+    assert len(cache) == 3
+    for key in cache:
+        assert cache[key] == 6
